import unicodedata


# 算出有几个非英文字符
def wide_chars(s):
    return sum(unicodedata.east_asian_width(x) == "W" for x in s)


# 非英文字符都算2个长度, 加上去
def width(s) -> int:
    return len(s) + wide_chars(s)


# 获取最长
def get_max_lens(rows: list):
    # 初始化为 0
    max_len = [0] * len(rows[0])
    for row in rows:
        for i in range(len(row)):
            # 要转 str, 避免错误
            str_width = width(str(row[i]))
            # print(row[i])
            # print(str_width)
            if str_width > max_len[i]:
                max_len[i] = str_width
    return max_len


def table(rows: list, spliter: str):
    max_lens = get_max_lens(rows)
    for row in rows:
        # formated_row = [str(row[i]).ljust(max_lens[i]) for i in range(len(row))]
        # ljust 使用 len 判断长度, 支持非英文字符, 导致对每个非英文字符多填充了一个空白
        # 解决办法是算出有n个非英字符, just 长度-n
        formated_row = [
            str(row[i]).ljust(max_lens[i] - wide_chars(str(row[i])))
            for i in range(len(row))
        ]
        print(spliter.join(formated_row))


if __name__ == "__main__":
    data = [
        ["用户名", "服务器地址", "服务器的一些说明"],
        ["用户名User", "Host", "Description"],
        ["User", "Host", "Description"],
        ["root", "h.bigzhu.net", "dump"],
        ["root", "racknerd.bigzhu.net", "racknerd"],
        ["bigzhu", "ssh.entube.app", "digitalocean"],
    ]
    table(data, " ")
    # i = "服务器地址什么"
    # j = "qwertyuiop"
    # print(width(i))
    # print(len(i))



from .cn_now import cn_now
import re


def test_now_print(capfd):
    """Check that User instance has the particular properties."""
    cn_now('BigZhu')
    out, err = capfd.readouterr()
    assert re.match(
        r'^(\d{4}-\d{2}-\d{2})\s+(\d{2}:\d{2}:\d{2}) BigZhu$', out) is not None
    assert err == ''



import telebot
from environs import Env
import sys
run_path = sys.path[0]
env = Env()
env.read_env(f'{run_path}/.env')  # read .env file, if it exists
TEL_TOKEN = env('TEL_TOKEN')
CHAT_ID = env('CHAT_ID')
bot = telebot.TeleBot(
    TEL_TOKEN, parse_mode=None
)




from zbig import ztime


# print whith zh_now time
def cn_now(str_in: str):
    t = ztime.cn_now()
    print(f"{t} {str_in}", flush=True)


if __name__ == "__main__":
    cn_now("Hello, World!")
    cn_now("你好，世界！")
    cn_now("こんにちは世界！")
    cn_now("안녕하세요 세계!")



import hashlib


def _tob(_string, enc="utf8"):
    if isinstance(_string, str):
        return _string.encode(enc)
    return b"" if _string is None else bytes(_string)


def args_hash(*args, **kw):
    """Calculate a hash value from string args and kwargs.
    Given the same positional arguments and keyword
    arguments, this function will produce the same
    hash key.
    """
    items = list(args) + [i for t in sorted(kw.items()) for i in t]
    items = ("__NoneType__" if _i is None else _i for _i in items)
    # All items must be strings
    args_string = "|".join(items)
    return hashlib.sha1(_tob(args_string)).hexdigest()



#!/usr/bin/env python3

import csv


def read_csv(file_path: str) -> tuple:
    rows = []
    with open(file_path, "r") as file:
        csvreader = csv.reader(file)
        header = next(csvreader)
        rows.extend(iter(csvreader))
    return header, rows


def write_csv_append(file_path: str, row: list):
    if check_duplicate(file_path, row):
        # https://docs.python.org/3/library/exceptions.html#exception-hierarchy
        raise ValueError(f"Duplicate data {row}")
    with open(file_path, "a") as file:
        # 默认是 windows 的换行结束符
        csvwriter = csv.writer(file, lineterminator="\n")
        csvwriter.writerow(row)


def check_duplicate(file_path: str, row: list) -> bool:
    _, rows = read_csv(file_path)
    return any(r == row for r in rows)


if __name__ == "__main__":
    header, rows = read_csv("hosts.csv")
    print(header)
    print(rows)
    for row in rows:
        write_csv_append("hosts.csv", row)
    print("write_csv done")
